@@ -212,7 +212,7 @@ XL_STANDARDWIDTH = 0x99
 XL_STRING = 0x207
 XL_STRING_B2 = 0x7
 XL_STYLE = 0x293
-XL_SUPBOOK = 0x1AE
+XL_SUPBOOK = 0x1AE # aka EXTERNALBOOK in OOo docs
 XL_TABLEOP = 0x236
 XL_TABLEOP2 = 0x37
 XL_TABLEOP_B2 = 0x36
